import ConfigParser
import trac
import time
import trac_fedmsg_plugin
import requests

if not 'get_request_object' in trac_fedmsg_plugin:
    import inspect
    def get_request_object():
        for frame in (f[0] for f in inspect.stack()):
            if 'req' in frame.f_locals:
                return frame.f_locals['req']


def recursive_str(dct):
    """ This function makes sure that dct is json serializable. """
    if not hasattr(dct, '__iter__'):
            return str(dct)
    for key in dct:
        if isinstance(dct[key], dict):
            dct[key] = recursive_str(dct[key])
        elif isinstance(dct[key], list):
            dct[key] = map(recursive_str, dct[key])
        elif isinstance(dct[key], unicode):
            dct[key] = str(dct[key].encode('utf-8'))
        else:
            # If it's not a dict or list, just run an str() over it
            dct[key] = str(dct[key])
    return dct


class BassetPlugin(trac_fedmsg_plugin.FedmsgPlugin):
    banned_fields = []

    def __init__(self, *args, **kwargs):
        # We explicitly do not initialize fedmsg
        trac.core.Component.__init__(self, *args, **kwargs)

        self.config = ConfigParser.SafeConfigParser()
        self.config.read(['trac-baseet.default.cfg',
                          '/etc/basset/trac-basset.default.cfg',
                          'trac-basset.cfg',
                          '/etc/basset/trac-basset.cfg'])

    def publish(self, topic, **msg):
        current_request = trac_fedmsg_plugin.get_request_object()
        msg['request_headers'] = get_request_object()._inheaders
        msg = recursive_str(msg)

        basset_request = {'action': 'trac.%s' % topic,
                          'time': int(time.time()),
                          'data': msg}

        url = self.config.get('submit', 'url')
        user = self.config.get('submit', 'username')
        pwd = self.config.get('submit', 'password')

        requests.post(url, auth=(user, pwd), json=basset_request)
